@@ -7,7 +7,7 @@
  *  UCRL-CODE-2003-006.
  *  
  *  This file is part of Scrub, a program for erasing disks.
- *  For details, see <http://www.llnl.gov/linux/scrub/>.
+ *  For details, see https://code.google.com/p/diskscrub/
  *  
  *  Scrub is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -24,13 +24,17 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 \*****************************************************************************/
 
-#define MAXPATBYTES 3
+#define MAXPATBYTES 16
 #define MAXSEQPATTERNS 35
-typedef enum { NORMAL, RANDOM, VERIFY } ptype_t;
+typedef enum {
+    PAT_NORMAL,
+    PAT_RANDOM,
+    PAT_VERIFY,
+} ptype_t;
 typedef struct {
     ptype_t     ptype;
     int         len;
-    int		pat[MAXPATBYTES];
+    int         pat[MAXPATBYTES];
 } pattern_t;
 
 typedef struct {
@@ -44,6 +48,8 @@ const sequence_t *seq_lookup(char *name);
 void              seq_list(void);
 char             *pat2str(pattern_t p);
 void              memset_pat(void *s, pattern_t p, size_t n);
+void              pattern_finalize(void);
+
 
 /*
  * vi:tabstop=4 shiftwidth=4 expandtab
