@@ -7,7 +7,7 @@
  *  UCRL-CODE-2003-006.
  *  
  *  This file is part of Scrub, a program for erasing disks.
- *  For details, see <http://www.llnl.gov/linux/scrub/>.
+ *  For details, see https://code.google.com/p/diskscrub/
  *  
  *  Scrub is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -55,11 +55,7 @@
 #include "sig.h"
 #include "pattern.h"
 
-/* NOTE: default blocksize was 8K in scrub 1.8, however on hpux
- * it was found that raising the default to 1M raised performance from
- * ~1.3 MB/s to 20MB/s. [Graham Smith]
- */
-#define BUFSIZE (1024*1024) /* default blocksize */
+#define BUFSIZE (4*1024*1024) /* default blocksize */
 
 static void       usage(void);
 static bool       scrub(char *path, off_t size, const sequence_t *seq,
@@ -76,12 +72,12 @@ static void       scrub_resfork(char *path, const sequence_t *seq,
 static void       scrub_disk(char *path, off_t size, const sequence_t *seq,
                       int bufsize, bool Sopt, bool sparse);
 
-#define OPTIONS "p:D:Xb:s:fSrvTLh"
+#define OPTIONS "p:D:Xb:s:fSrvTLRh"
 #if HAVE_GETOPT_LONG
 #define GETOPT(ac,av,opt,lopt) getopt_long(ac,av,opt,lopt,NULL)
 static struct option longopts[] = {
     {"pattern",          required_argument,  0, 'p'},
-    {"dirent",           no_argument,        0, 'D'},
+    {"dirent",           required_argument,  0, 'D'},
     {"freespace",        no_argument,        0, 'X'},
     {"blocksize",        required_argument,  0, 'b'},
     {"device-size",      required_argument,  0, 's'},
@@ -91,6 +87,7 @@ static struct option longopts[] = {
     {"version",          no_argument,        0, 'v'},
     {"test-sparse",      no_argument,        0, 'T'},
     {"no-link",          no_argument,        0, 'L'},
+    {"no-hwrand",        no_argument,        0, 'R'},
     {"help",             no_argument,        0, 'h'},
     {0, 0, 0, 0},
 };
@@ -114,6 +111,7 @@ main(int argc, char *argv[])
     bool ropt = false;
     bool Topt = false;
     bool Lopt = false;
+    bool Ropt = false;
     extern int optind;
     extern char *optarg;
     int c;
@@ -170,6 +168,9 @@ main(int argc, char *argv[])
         case 'L':   /* --no-link */
             Lopt = true;
             break;
+        case 'R':   /* --no-hwrand */
+            Ropt = true;
+            break;
         case 'h':   /* --help */
         default:
             usage();
@@ -184,11 +185,14 @@ main(int argc, char *argv[])
     assert(seq != NULL);
     printf("%s: using %s patterns\n", prog, seq->desc);
 
+    if (Ropt)
+        disable_hwrand();
+
     /* Handle -X specially.
      */
     if (Xopt) {
-        if (filetype(filename) != NOEXIST) {
-            fprintf(stderr, "%s: -X argument cannot exist\n", prog);
+        if (filetype(filename) != FILE_NOEXIST) {
+            fprintf(stderr, "%s: -X directory already exists\n", prog);
             exit(1);
         }
         if (Dopt) {
@@ -196,18 +200,18 @@ main(int argc, char *argv[])
             exit(1);
         }
         scrub_free(filename, sopt, seq, bopt, Sopt);
-        exit(0);
+        goto done;
     } 
 
     switch (filetype(filename)) {
-        case NOEXIST:
+        case FILE_NOEXIST:
             fprintf(stderr, "%s: %s does not exist\n", prog, filename);
             exit(1);
-        case OTHER:
+        case FILE_OTHER:
             fprintf(stderr, "%s: %s is wrong type of file\n", prog, filename);
             exit(1);
-        case BLOCK:
-        case CHAR:
+        case FILE_BLOCK:
+        case FILE_CHAR:
             if (Dopt) {
                 fprintf(stderr, "%s: cannot use -D with special file\n", prog);
                 exit(1);
@@ -227,7 +231,7 @@ main(int argc, char *argv[])
             }
             scrub_disk(filename, sopt, seq, bopt, Sopt, Topt);
             break;
-        case LINK:
+        case FILE_LINK:
             if (Lopt) {
                 if (ropt) {
                     printf("%s: unlinking %s\n", prog, filename);
@@ -239,7 +243,7 @@ main(int argc, char *argv[])
                 }
                 break;
             }
-        case REGULAR:
+        case FILE_REGULAR:
             if (access(filename, R_OK|W_OK) < 0) {
                 fprintf(stderr, "%s: no rw access to %s\n", prog, filename);
                 exit(1);
@@ -272,24 +276,30 @@ main(int argc, char *argv[])
             }
             break;
     }
+done:
+    pattern_finalize ();
     exit(0);
 }
 
 static void 
 usage(void)
 {
-    fprintf(stderr, "Usage: %s [OPTIONS] file\n%s%s%s%s%s%s%s%s%s%s", prog,
-"  -v, --version           display scrub version and exit\n",
-"  -p, --pattern pat       select scrub pattern sequence\n",
-"  -b, --blocksize size    set I/O buffer size (default 1m)\n",
-"  -s, --device-size size  set device size manually\n",
-"  -X, --freespace         create dir+files, fill until ENOSPC, then scrub\n",
-"  -D, --dirent newname    after scrubbing file, scrub dir entry, rename\n",
-"  -f, --force             scrub despite signature from previous scrub\n",
-"  -S, --no-signature      do not write scrub signature after scrub\n",
-"  -r, --remove            remove file after scrub\n",
-"  -L, --no-link           do not scrub link target\n",
-"  -h, --help              display this help message\n");
+    fprintf(stderr,
+"Usage: %s [OPTIONS] file\n"
+"  -v, --version           display scrub version and exit\n"
+"  -p, --pattern pat       select scrub pattern sequence\n"
+"  -b, --blocksize size    set I/O buffer size (default 4m)\n"
+"  -s, --device-size size  set device size manually\n"
+"  -X, --freespace dir     create dir+files, fill until ENOSPC, then scrub\n"
+"  -D, --dirent newname    after scrubbing file, scrub dir entry, rename\n"
+"  -f, --force             scrub despite signature from previous scrub\n"
+"  -S, --no-signature      do not write scrub signature after scrub\n"
+"  -r, --remove            remove file after scrub\n"
+"  -L, --no-link           do not scrub link target\n"
+"  -R, --no-hwrand         do not use a hardware random number generator\n"
+"  -h, --help              display this help message\n"
+    , prog);
+
     fprintf(stderr, "Available patterns are:\n");
     seq_list ();
     exit(1);
@@ -311,7 +321,7 @@ scrub(char *path, off_t size, const sequence_t *seq, int bufsize,
     bool isfull = false;
     off_t written;
 
-    if (!(buf = malloc(bufsize))) {
+    if (!(buf = alloc_buffer(bufsize))) {
         fprintf(stderr, "%s: out of memory\n", prog);
         exit(1);
     }
@@ -324,7 +334,7 @@ scrub(char *path, off_t size, const sequence_t *seq, int bufsize,
         if (i > 0)
             enospc = false;
         switch (seq->pat[i].ptype) {
-            case RANDOM:
+            case PAT_RANDOM:
                 printf("%s: %-8s", prog, "random");
                 progress_create(&p, 50);
                 churnrand();
@@ -333,7 +343,7 @@ scrub(char *path, off_t size, const sequence_t *seq, int bufsize,
                                    (refill_t)genrand, sparse, enospc);
                 progress_destroy(p);
                 break;
-            case NORMAL:
+            case PAT_NORMAL:
                 printf("%s: %-8s", prog, pat2str(seq->pat[i]));
                 progress_create(&p, 50);
                 memset_pat(buf, seq->pat[i], bufsize);
@@ -342,7 +352,7 @@ scrub(char *path, off_t size, const sequence_t *seq, int bufsize,
                                    NULL, sparse, enospc);
                 progress_destroy(p);
                 break;
-            case VERIFY:
+            case PAT_VERIFY:
                 printf("%s: %-8s", prog, pat2str(seq->pat[i]));
                 progress_create(&p, 50);
                 memset_pat(buf, seq->pat[i], bufsize);
@@ -362,6 +372,11 @@ scrub(char *path, off_t size, const sequence_t *seq, int bufsize,
             assert(enospc == true);
             isfull = true; 
             size = written;
+            if (size == 0) {
+                fprintf(stderr, "%s: file system is full (0 bytes written)\n",
+                        prog);
+                break;
+            }
         }
     }
     if (!Sopt)
@@ -404,7 +419,6 @@ static void
 scrub_free(char *dirpath, off_t size, const sequence_t *seq, 
            int bufsize, bool Sopt)
 {
-    char *buf;
     char path[MAXPATHLEN];
     int fileno = 0;
     struct stat sb;
@@ -414,6 +428,7 @@ scrub_free(char *dirpath, off_t size, const sequence_t *seq,
         fprintf(stderr, "%s: mkdir %s: %s\n", prog, path, strerror(errno));
         exit(1);
     } 
+    fprintf (stderr, "%s: created directory %s\n", prog, dirpath);
     if (stat(dirpath, &sb) < 0) {
         fprintf(stderr, "%s: stat %s: %s\n", prog, path, strerror(errno));
         exit(1);
@@ -425,11 +440,21 @@ scrub_free(char *dirpath, off_t size, const sequence_t *seq,
     if (size == 0)
         size = 1024*1024*1024;
     size = blkalign(size, sb.st_blksize, DOWN);
-
     do {
         snprintf(path, sizeof(path), "%s/scrub.%.3d", dirpath, fileno++);
         isfull = scrub(path, size, seq, bufsize, Sopt, false, true);
     } while (!isfull);
+    while (--fileno >= 0) {
+        snprintf(path, sizeof(path), "%s/scrub.%.3d", dirpath, fileno);
+        if (unlink(path) < 0)
+            fprintf(stderr, "%s: unlink %s: %s\n", prog, path, strerror(errno));
+        else
+            fprintf(stderr, "%s: unlinked %s\n", prog, path);
+    }
+    if (rmdir(dirpath) < 0)
+        fprintf(stderr, "%s: rmdir %s: %s\n", prog, dirpath, strerror(errno));
+    else
+        fprintf(stderr, "%s: removed %s\n", prog, dirpath);
 }
 
 /* Scrub name component of a directory entry through succesive renames.
@@ -443,12 +468,12 @@ scrub_dirent(char *path, char *newpath)
     filetype_t ftype = filetype(path);
 
     assert(seq != NULL);
-    assert(ftype == REGULAR);
+    assert(ftype == FILE_REGULAR);
 
     printf("%s: scrubbing directory entry\n", prog);
 
     for (i = 0; i < seq->len; i++) {
-        assert(seq->pat[i].ptype == REGULAR);
+        assert(seq->pat[i].ptype == PAT_NORMAL);
         assert(seq->pat[i].len == 1);
         printf("%s: %-8s", prog, pat2str(seq->pat[i]));
         progress_create(&p, 50);
@@ -471,7 +496,7 @@ scrub_file(char *path, off_t size, const sequence_t *seq,
     struct stat sb;
     filetype_t ftype = filetype(path);
 
-    assert(ftype == REGULAR || ftype == LINK);
+    assert(ftype == FILE_REGULAR || ftype == FILE_LINK);
 
     if (stat(path, &sb) < 0) {
         fprintf(stderr, "%s: stat %s: %s\n", prog, path, strerror(errno));
@@ -505,7 +530,7 @@ scrub_resfork(char *path, const sequence_t *seq, int bufsize)
     off_t rsize; 
     filetype_t ftype = filetype(path);
 
-    assert(ftype == REGULAR);
+    assert(ftype == FILE_REGULAR);
     (void)snprintf(rpath, sizeof(rpath), "%s/..namedfork/rsrc", path);
     if (stat(rpath, &rsb) < 0)
         return;
@@ -531,7 +556,7 @@ scrub_disk(char *path, off_t size, const sequence_t *seq, int bufsize,
 {
     filetype_t ftype = filetype(path);
 
-    assert(ftype == BLOCK || ftype == CHAR);
+    assert(ftype == FILE_BLOCK || ftype == FILE_CHAR);
     if (size == 0) {
         size = getsize(path);
         if (size == 0) {
