@@ -7,7 +7,7 @@
  *  UCRL-CODE-2003-006.
  *  
  *  This file is part of Scrub, a program for erasing disks.
- *  For details, see <http://www.llnl.gov/linux/scrub/>.
+ *  For details, see https://code.google.com/p/diskscrub/
  *  
  *  Scrub is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -29,8 +29,10 @@
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <errno.h>
 #include <unistd.h>
 #include <libgen.h>
+#include <stdlib.h>
 
 #include "util.h"
 
@@ -77,21 +79,21 @@ filetype(char *path)
 {
     struct stat sb;
 
-    filetype_t res = NOEXIST;
+    filetype_t res = FILE_NOEXIST;
 
     if (lstat(path, &sb) == 0 && S_ISLNK(sb.st_mode)) {
-        return LINK;
+        return FILE_LINK;
     }
 
     if (stat(path, &sb) == 0) {
         if (S_ISREG(sb.st_mode))
-            res = REGULAR;
+            res = FILE_REGULAR;
         else if (S_ISCHR(sb.st_mode))
-            res = CHAR;
+            res = FILE_CHAR;
         else if (S_ISBLK(sb.st_mode))
-            res = BLOCK;
+            res = FILE_BLOCK;
         else
-            res = OTHER;
+            res = FILE_OTHER;
     }
     return res;
 }
@@ -117,6 +119,29 @@ blkalign(off_t offset, int blocksize, round_t rtype)
     return offset;
 }
 
+/* Allocate an aligned buffer
+ */
+#define ALIGNMENT	(16*1024*1024) /* Hopefully good enough */
+void *
+alloc_buffer(int bufsize)
+{
+    void *ptr;
+
+#ifdef HAVE_POSIX_MEMALIGN
+    int err = posix_memalign(&ptr, ALIGNMENT, bufsize);
+    if (err) {
+	errno = err;
+	ptr = NULL;
+    }
+#elif defined(HAVE_MEMALIGN)
+    ptr = memalign(ALIGNMENT, bufsize);
+#else
+    ptr = malloc(bufsize);	/* Hope for the best? */
+#endif
+
+    return ptr;
+}
+
 /*
  * vi:tabstop=4 shiftwidth=4 expandtab
  */
